#define HAVE_SOCKLEN_T
#define HAVE_STDINT_H

#include <stdio.h>
#include <CoreServices/CoreServices.h>
#include <CoreAudio/CoreAudio.h>
#include <AudioUnit/AudioUnit.h>
#include <AudioToolbox/AudioToolbox.h>
#include <unistd.h>
#include <pthread.h>
#include <faad.h>
#include <mp4.h>

#define BUFFER_SIZE	FAAD_MIN_STREAMSIZE*64

OSStatus MyFileRenderProc(void *inRefCon, AudioUnitRenderActionFlags    *inActionFlags,
	const AudioTimeStamp *inTimeStamp, UInt32 inBusNumber,
	UInt32 inNumFrames, AudioBufferList *ioData);
OSStatus MyACComplexInputProc   (AudioConverterRef                        inAudioConverter,
	UInt32                               *ioNumberDataPackets,
	AudioBufferList                                   *ioData,
	AudioStreamPacketDescription  **outDataPacketDescription,
	void*                                          inUserData);

static AudioUnit outputUnit;
static AudioConverterRef converter;
static MP4FileHandle mp4file;
static faacDecHandle	decoder;
static unsigned char *buffer1;
void *gSourceBuffer;
static int numSamples;
static int mp4track;
static int sampleID;
static unsigned int bufferSize;
static unsigned long buffervalid, bufferconsumed;
static int totalSize;
static int playedSize;
static int totalTime;
static int currentTime;
static int lastBuffer;
static int playDone;
static int isMP4;
static FILE *fp;
static unsigned long samplerate;
static unsigned char channels;

const u_int8_t mp4AudioTypes[] =
  {
    MP4_MPEG1_AUDIO_TYPE,		// 0x6B
    MP4_MPEG2_AUDIO_TYPE,		// 0x69
    MP4_MPEG2_AAC_MAIN_AUDIO_TYPE,	// 0x66
    MP4_MPEG2_AAC_LC_AUDIO_TYPE,	// 0x67
    MP4_MPEG2_AAC_SSR_AUDIO_TYPE,	// 0x68
    MP4_MPEG4_AUDIO_TYPE,		// 0x40
    0
	};
const char *mpeg4AudioNames[]=
  {
    "!!!!MPEG-4 Audio track Invalid !!!!!!!",
    "MPEG-4 AAC Main profile",
    "MPEG-4 AAC Low Complexity profile",
    "MPEG-4 AAC SSR profile",
    "MPEG-4 AAC Long Term Prediction profile",
    "MPEG-4 AAC Scalable",
    "MPEG-4 CELP",
    "MPEG-4 HVXC",
    "MPEG-4 Text To Speech",
    "MPEG-4 Main Synthetic profile",
    "MPEG-4 Wavetable Synthesis profile",
    "MPEG-4 MIDI Profile",
    "MPEG-4 Algorithmic Synthesis and Audio FX profile"
};
const char *mp4AudioNames[]=
  {
    "MPEG-1 Audio Layers 1,2 or 3",
    "MPEG-2 low biterate (MPEG-1 extension) - MP3",
    "MPEG-2 AAC Main Profile",
    "MPEG-2 AAC Low Complexity profile",
    "MPEG-2 AAC SSR profile",
    "MPEG-4 audio (MPEG-4 AAC)",
    0
	};

struct seek_list
{
    struct seek_list *next;
    long long offset;
};

typedef struct state
{
    /* general stuff */
    faacDecHandle hDecoder;
    int samplerate;
    unsigned char channels;
    double decode_pos_ms; // current decoding position, in milliseconds
    int paused; // are we paused?
    int seek_needed; // if != -1, it is the point that the decode thread should seek to, in ms.
    char filename[256];
    int filetype; /* 0: MP4; 1: AAC */
    int last_frame;
    long long last_offset;

    /* MP4 stuff */
    MP4FileHandle mp4file;
    int mp4track;
    MP4SampleId numSamples;
    MP4SampleId sampleId;

    /* AAC stuff */
    FILE *aacfile;
    long m_aac_bytes_into_buffer;
    long m_aac_bytes_consumed;
    long long m_file_offset;
    unsigned char *m_aac_buffer;
    int m_at_eof;
    double cur_pos_sec;
    int m_header_type;
    struct seek_list *m_head;
    struct seek_list *m_tail;
    unsigned long m_length;

    /* for gapless decoding */
    unsigned int useAacLength;
    unsigned int framesize;
    unsigned int initial;
    unsigned long timescale;
} state;


int getAACTrack(MP4FileHandle file)
{
  int numTracks = MP4GetNumberOfTracks(file, NULL, 0);
  int i=0;

  for(i=0;i<numTracks;i++){
    MP4TrackId trackID = MP4FindTrackId(file, i, NULL, 0);
    const char *trackType = MP4GetTrackType(file, trackID);
    if(!strcmp(trackType, MP4_AUDIO_TRACK_TYPE)){//we found audio track !
      int j=0;
      u_int8_t audiotype = MP4GetTrackAudioType(file, trackID);
      while(mp4AudioTypes[j]){ // what kind of audio is ?
	if(mp4AudioTypes[j] == audiotype){
	  if(mp4AudioTypes[j] == MP4_MPEG4_AUDIO_TYPE){//MPEG4 audio ok
	    audiotype = MP4GetTrackAudioMpeg4Type(file, trackID);
	    printf("%d-%s\n", audiotype, mpeg4AudioNames[audiotype]);
	    return (trackID);
	  }
	  else{
	    printf("%s\n", mp4AudioNames[j]);
	    if (mp4AudioTypes[j]== MP4_MPEG2_AAC_LC_AUDIO_TYPE ||
		mp4AudioTypes[j]== MP4_MPEG2_AAC_MAIN_AUDIO_TYPE ||
		mp4AudioTypes[j]== MP4_MPEG2_AAC_SSR_AUDIO_TYPE)
	      return(trackID);
	    return(-1);
	  }
	}
	j++;
      }
    }
  }
    return(-1);
}

int file_length(FILE *f)
{
    long end = 0;
    long cur = ftell(f);
    fseek(f, 0, SEEK_END);
    end = ftell(f);
    fseek(f, cur, SEEK_SET);

    return end;
}

int fill_buffer(state *st)
{
    int bread;

    if (st->m_aac_bytes_consumed > 0)
    {
        if (st->m_aac_bytes_into_buffer)
        {
            memmove((void*)st->m_aac_buffer, (void*)(st->m_aac_buffer + st->m_aac_bytes_consumed),
                st->m_aac_bytes_into_buffer*sizeof(unsigned char));
        }

        if (!st->m_at_eof)
        {
            bread = fread((void*)(st->m_aac_buffer + st->m_aac_bytes_into_buffer),
                1, st->m_aac_bytes_consumed, st->aacfile);

            if (bread != st->m_aac_bytes_consumed)
                st->m_at_eof = 1;

            st->m_aac_bytes_into_buffer += bread;
        }

        st->m_aac_bytes_consumed = 0;

        if (st->m_aac_bytes_into_buffer > 3)
        {
            if (memcmp(st->m_aac_buffer, "TAG", 3) == 0)
                st->m_aac_bytes_into_buffer = 0;
        }
        if (st->m_aac_bytes_into_buffer > 11)
        {
            if (memcmp(st->m_aac_buffer, "LYRICSBEGIN", 11) == 0)
                st->m_aac_bytes_into_buffer = 0;
        }
        if (st->m_aac_bytes_into_buffer > 8)
        {
            if (memcmp(st->m_aac_buffer, "APETAGEX", 8) == 0)
                st->m_aac_bytes_into_buffer = 0;
        }
    }

    return 1;
}

void advance_buffer(state *st, int bytes)
{
    st->m_file_offset += bytes;
    st->m_aac_bytes_consumed = bytes;
    st->m_aac_bytes_into_buffer -= bytes;
}

int adts_parse(state *st, long long *bitrate, double *length)
{
    static int sample_rates[] = {96000,88200,64000,48000,44100,32000,24000,22050,16000,12000,11025,8000};
    int frames, frame_length;
    int t_framelength = 0;
    int samplerates;
    double frames_per_sec, bytes_per_frame;

    /* Read all frames to ensure correct time and bitrate */
    for (frames = 0; /* */; frames++)
    {
        fill_buffer(st);

        if (st->m_aac_bytes_into_buffer > 7)
        {
            /* check syncword */
            if (!((st->m_aac_buffer[0] == 0xFF)&&((st->m_aac_buffer[1] & 0xF6) == 0xF0)))
                break;

            st->m_tail->offset = st->m_file_offset;
            st->m_tail->next = (struct seek_list*)malloc(sizeof(struct seek_list));
            st->m_tail = st->m_tail->next;
            st->m_tail->next = NULL;

            if (frames == 0)
                samplerates = sample_rates[(st->m_aac_buffer[2]&0x3c)>>2];

            frame_length = ((((unsigned int)st->m_aac_buffer[3] & 0x3)) << 11)
                | (((unsigned int)st->m_aac_buffer[4]) << 3) | (st->m_aac_buffer[5] >> 5);

            t_framelength += frame_length;

            if (frame_length > st->m_aac_bytes_into_buffer)
                break;

            advance_buffer(st, frame_length);
        } else {
            break;
        }
    }

    frames_per_sec = (double)samplerates/1024.0;
    if (frames != 0)
        bytes_per_frame = (double)t_framelength/(double)(frames*1000);
    else
        bytes_per_frame = 0;
    *bitrate = (long long)(8. * bytes_per_frame * frames_per_sec + 0.5);
    if (frames_per_sec != 0)
        *length = (double)frames/frames_per_sec;
    else
        *length = 1;

    return 1;
}

int getsonglength(const char *fn)
{
    long msDuration = 0;

	int tagsize = 0;
	int bread = 0;
	double length = 0.;
	long long bitrate = 128;
	struct seek_list *target;
	state len_state;
	
	memset(&len_state, 0, sizeof(state));
	
	if (!(len_state.aacfile = fopen(fn, "rb")))
	{
		// error
		return 0;
	}
	
	len_state.m_at_eof = 0;
	
	if (!(len_state.m_aac_buffer = (unsigned char*)malloc(768*6)))
	{
		//console::error("Memory allocation error.", "foo_mp4");
		return 0;
	}
	memset(len_state.m_aac_buffer, 0, 768*6);
	
	bread = fread(len_state.m_aac_buffer, 1, 768*6, len_state.aacfile);
	len_state.m_aac_bytes_into_buffer = bread;
	len_state.m_aac_bytes_consumed = 0;
	len_state.m_file_offset = 0;
	
	if (bread != 768*6)
		len_state.m_at_eof = 1;
	
	if (!memcmp(len_state.m_aac_buffer, "ID3", 3))
	{
		/* high bit is not used */
		tagsize = (len_state.m_aac_buffer[6] << 21) | (len_state.m_aac_buffer[7] << 14) |
			(len_state.m_aac_buffer[8] <<  7) | (len_state.m_aac_buffer[9] <<  0);
		
		tagsize += 10;
		advance_buffer(&len_state, tagsize);
	}
	
	len_state.m_head = (struct seek_list*)malloc(sizeof(struct seek_list));
	len_state.m_tail = len_state.m_head;
	len_state.m_tail->next = NULL;
	
	len_state.m_header_type = 0;
	if ((len_state.m_aac_buffer[0] == 0xFF) && ((len_state.m_aac_buffer[1] & 0xF6) == 0xF0))
	{
		if (1) //(m_reader->can_seek())
		{
			adts_parse(&len_state, &bitrate, &length);
			fseek(len_state.aacfile, tagsize, SEEK_SET);
			
			bread = fread(len_state.m_aac_buffer, 1, 768*6, len_state.aacfile);
			if (bread != 768*6)
				len_state.m_at_eof = 1;
			else
				len_state.m_at_eof = 0;
			len_state.m_aac_bytes_into_buffer = bread;
			len_state.m_aac_bytes_consumed = 0;
			
			len_state.m_header_type = 1;
		}
	} else if (memcmp(len_state.m_aac_buffer, "ADIF", 4) == 0) {
		int skip_size = (len_state.m_aac_buffer[4] & 0x80) ? 9 : 0;
		bitrate = ((unsigned int)(len_state.m_aac_buffer[4 + skip_size] & 0x0F)<<19) |
			((unsigned int)len_state.m_aac_buffer[5 + skip_size]<<11) |
			((unsigned int)len_state.m_aac_buffer[6 + skip_size]<<3) |
			((unsigned int)len_state.m_aac_buffer[7 + skip_size] & 0xE0);
		
		length = (double)file_length(len_state.aacfile);
		if (length == -1)
			length = 0;
		else
			length = ((double)length*8.)/((double)bitrate) + 0.5;
		
		len_state.m_header_type = 2;
	} else {
		length = (double)file_length(len_state.aacfile);
		length = ((double)length*8.)/((double)bitrate*1000.) + 0.5;
		
		len_state.m_header_type = 0;
	}
	
	if (len_state.m_aac_buffer)
		free(len_state.m_aac_buffer);
	
	target = len_state.m_head;
	while (target)
	{
		struct seek_list *tmp = target;
		target = target->next;
		if (tmp) free(tmp);
	}
	
	fclose(len_state.aacfile);
	
	return (int)(length*1000.);
}

int main(int argc, char *argv[])
{
	OSStatus	err ;
	AudioStreamBasicDescription inFormat;
	AudioStreamBasicDescription outFormat;
	AURenderCallbackStruct  renderCallback;
	Boolean outWritable;
	UInt32		count ;
	int i=0;
	char opt, buf[4];
	extern char *optarg ;
	extern int optind;
	
	while((opt = getopt(argc,argv,"o:t:f:Fm:r:s:i:c:v:")) != -1) {
		switch(opt) {
		  case 'o' :
			break;
		  case 't' :
			break;
		  case 'f' :
			break;
		  case 'F' :
			break;
		  case 'm' :
			break;
		  case 'r' :
			break;
		  case 's' :
			break;
		  case 'i' :
			break;
		  case 'c' :
			break;
		  case 'v' :
			break;
		}
		i++;
	}
	
	
	if(argc <= i+1) {
		printf("usage: %s <SPC file>\n",argv[0]);
		return 0;
	}
	
	if(!(fp=fopen(argv[optind],"rb"))) {
		fprintf(stderr,"can't open file.\n");
		return 0;
	}
	
	fclose(fp);
	
	ComponentDescription desc;
	Component comp;
	AudioUnitConnection	connection;
	connection.sourceOutputNumber = 0;
	connection.destInputNumber    = 0;
	
	/* Open default output nuit */
	desc.componentType = kAudioUnitType_Output; 
	desc.componentSubType = kAudioUnitSubType_DefaultOutput;
	desc.componentManufacturer = kAudioUnitManufacturer_Apple;
	desc.componentFlags = 0;
	desc.componentFlagsMask = 0;
	comp = FindNextComponent(NULL, &desc);
	if (comp == NULL) {
		fprintf(stderr, "can't find default output unit\n");
		return 0;
	}
	
	err = OpenAComponent(comp, &outputUnit);
	if (err)  {
		fprintf(stderr, "can't open default output unit\n");
		return 0;
	}
	
	err = AudioUnitInitialize(outputUnit);
	if(err != noErr) {
		fprintf(stderr, "AudioUnitInitialize failed.\n");
		return 0;
	}
	
	err = AudioUnitGetPropertyInfo(outputUnit, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Output, 0, &count, &outWritable);
	err = AudioUnitGetProperty (outputUnit, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Output, 0, &outFormat, &count);
	if(err != noErr) {
		fprintf(stderr, "AudioUnitGetProperty failed.\n");
		return 0;
	}
	AudioUnitSetProperty (outputUnit, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Input, 0, &outFormat, count);
	if(err != noErr) {
		fprintf(stderr, "AudioUnitSetProperty failed.\n");
		return 0;
	}
	
	if(!(mp4file = MP4Read(argv[optind], 0))){
		isMP4 = 0;
		MP4Close(mp4file);
		
		totalTime = getsonglength(argv[optind])/1000;
		totalSize = totalTime * 44100 * 8;
		if(!totalTime) return 0;
		
		if(!(fp=fopen(argv[optind],"rb"))) {
			fprintf(stderr,"can't open file.\n");
			return 0;
		}
		decoder = faacDecOpen();
		faacDecConfigurationPtr config = faacDecGetCurrentConfiguration(decoder);
		config->useOldADTSFormat = 0;
		config->outputFormat = 4;
		faacDecSetConfiguration(decoder, config);
		if((buffer1 = malloc(BUFFER_SIZE)) == NULL) {
			fclose(fp);
			return 0;
		}
		if((buffervalid = fread(buffer1, 1, BUFFER_SIZE, fp))==0){
			fclose(fp);
			return 0;
		}
		if(!strncmp(buffer1, "ID3", 3)) {
			int size = 0;
			fseek(fp, 0, SEEK_SET);
			size = (buffer1[6]<<21) | (buffer1[7]<<14) | (buffer1[8]<<7) | buffer1[9];
			size+=10;
			fread(buffer1, 1, size, fp);
			buffervalid = fread(buffer1, 1, BUFFER_SIZE, fp);
		}
		bufferconsumed = faacDecInit(decoder, buffer1, buffervalid, &samplerate, &channels);
	}
	else {
		isMP4 = 1;
		if((mp4track = getAACTrack(mp4file)) < 0){
			MP4Close(mp4file);
			return 0;
		}
		
		decoder = faacDecOpen();
		MP4GetTrackESConfiguration(mp4file, mp4track, &buffer1, &bufferSize);
		if(!buffer1){
			return 0;
		}
		faacDecConfigurationPtr config = faacDecGetCurrentConfiguration(decoder);
		config->outputFormat = 4;
		faacDecSetConfiguration(decoder, config);
		
		if(faacDecInit2(decoder, buffer1, bufferSize, &samplerate, &channels)<0){
			return 0;
		}
		MP4Duration duration = MP4GetTrackDuration(mp4file, mp4track);
		totalTime = MP4ConvertFromTrackDuration(mp4file, mp4track, duration, MP4_SECS_TIME_SCALE);
		totalSize = totalTime * 44100 * 8;
		
		numSamples = MP4GetTrackNumberOfSamples(mp4file, mp4track);
	}
	
	inFormat.mSampleRate = samplerate;
	inFormat.mBytesPerPacket = 8;
	inFormat.mFramesPerPacket = 1;
	inFormat.mBytesPerFrame = 8;
	inFormat.mChannelsPerFrame = channels;
	inFormat.mBitsPerChannel = 32;
	inFormat.mFormatID = kAudioFormatLinearPCM;
	inFormat.mFormatFlags = kLinearPCMFormatFlagIsFloat | kLinearPCMFormatFlagIsPacked | kLinearPCMFormatFlagIsBigEndian;
	
	err = AudioConverterNew(&inFormat, &outFormat, &converter);
	if(err != noErr) {
		fprintf(stderr, "AudioConverterNew failed.\n");
		return 0;
	}
	
	memset(&renderCallback, 0, sizeof(AURenderCallbackStruct));
	renderCallback.inputProc = MyFileRenderProc;
	renderCallback.inputProcRefCon =0;
	err = AudioUnitSetProperty (outputUnit, kAudioUnitProperty_SetRenderCallback, kAudioUnitScope_Input, 0, &renderCallback, sizeof(AURenderCallbackStruct));
	if(err != noErr) {
		fprintf(stderr, "AudioUnitSetProperty failed.\n");
		return 0;
	}
	
	
	//printf("%f, %f\n",totalTime, fadeOut);
	
	/*
	if((buffer1 = (char *)malloc(SOFTBUFSIZE)) == NULL) {
		fprintf(stderr,"malloc error\n");
		return 0;
	}
	if((buffer2 = (char *)malloc(SOFTBUFSIZE)) == NULL) {
		fprintf(stderr,"malloc error\n");
		return 0;
	}
	*/
	sampleID = 1;
	lastBuffer = 0;
	playDone = 0;
	playedSize = 0;
	currentTime = -1;
	
	AudioOutputUnitStart(outputUnit);
	
	while(!playDone) {
		usleep(10000);
	}
	
	AudioConverterDispose(converter);
	AudioOutputUnitStop(outputUnit);
	AudioUnitUninitialize (outputUnit);
	CloseComponent(outputUnit);
	return 0;
}

OSStatus MyFileRenderProc(void *inRefCon, AudioUnitRenderActionFlags    *inActionFlags,
                        const AudioTimeStamp *inTimeStamp, UInt32 inBusNumber,
                        UInt32 inNumFrames, AudioBufferList *ioData)
{
	OSStatus err= noErr;
	void *inInputDataProcUserData=NULL;
	AudioStreamPacketDescription* outPacketDescription =NULL;
	
	err = AudioConverterFillComplexBuffer(converter, MyACComplexInputProc ,inInputDataProcUserData , &inNumFrames, ioData, outPacketDescription);
	
	return err;
}

OSStatus MyACComplexInputProc   (AudioConverterRef                        inAudioConverter,
                                 UInt32                               *ioNumberDataPackets,
                                 AudioBufferList                                   *ioData,
                                 AudioStreamPacketDescription  **outDataPacketDescription,
                                 void*                                          inUserData)
{
	OSStatus    err = noErr;
	int i, percent, ret;
	faacDecFrameInfo	frameInfo;
	
	ioData->mBuffers[0].mData = NULL;      
	//ioData->mBuffers[0].mDataByteSize = 0;
	
	if(lastBuffer) {
		playDone = 1;
		return noErr;
	}
	/*
	if(!buffer2Rest) {
		while(!bufferIsFull) usleep(10000);
		memcpy(buffer2, buffer1, SOFTBUFSIZE);
		buffer1Rest = SOFTBUFSIZE;
		buffer1Top = buffer1;
		bufferIsFull = 0;
		buffer2Rest = SOFTBUFSIZE;
	}
	*/
	if(isMP4) {
		buffer1 = NULL;
		bufferSize = 0;
		
		ret = MP4ReadSample(mp4file, mp4track, sampleID++, &buffer1, &bufferSize, NULL, NULL, NULL, NULL);
		if((ret==0) || (buffer1 == NULL)){
			lastBuffer = 1;
			return noErr;
		}
		gSourceBuffer = faacDecDecode(decoder, &frameInfo, buffer1, bufferSize);
		if(frameInfo.error > 0){
			lastBuffer = 1;
			return noErr;
		}
		if(buffer1) {
			free(buffer1); buffer1=NULL; bufferSize=0;
		}
		//printf("%d,%d\n",frameInfo.bytesconsumed, frameInfo.samples);
		*ioNumberDataPackets = frameInfo.samples/2;
		ioData->mBuffers[0].mData = gSourceBuffer;
		ioData->mBuffers[0].mDataByteSize = frameInfo.samples*4;
		
		if(sampleID > numSamples) {
			lastBuffer = 1;
		}
		playedSize += ioData->mBuffers[0].mDataByteSize;
	}
	else {
		if(bufferconsumed > 0){
			memmove(buffer1, &buffer1[bufferconsumed], buffervalid-bufferconsumed);
			buffervalid -= bufferconsumed;
			buffervalid += fread(&buffer1[buffervalid], 1, BUFFER_SIZE-buffervalid, fp);
			bufferconsumed = 0;
		}
		gSourceBuffer = faacDecDecode(decoder, &frameInfo, buffer1, buffervalid);
		if(frameInfo.error){
			faacDecConfigurationPtr config = faacDecGetCurrentConfiguration(decoder);
			if(config->useOldADTSFormat != 1) {
				faacDecClose(decoder);
				decoder = faacDecOpen();
				config = faacDecGetCurrentConfiguration(decoder);
				config->useOldADTSFormat = 1;
				faacDecSetConfiguration(decoder, config);
				faacDecInit(decoder, buffer1, buffervalid, &samplerate, &channels);
				return noErr;
			}
			else {
				lastBuffer = 1;
				return noErr;
			}
		}
		bufferconsumed += frameInfo.bytesconsumed;
		*ioNumberDataPackets = frameInfo.samples/2;
		ioData->mBuffers[0].mData = gSourceBuffer;
		ioData->mBuffers[0].mDataByteSize = frameInfo.samples*4;
		
		playedSize += ioData->mBuffers[0].mDataByteSize;
		if(playedSize >= totalSize) {
			lastBuffer = 1;
			playedSize = totalSize;
		}
	}
	int secint = (int)playedSize / 44100 / 8;
	
	if(secint != currentTime) {
		percent = (int)(100.0*playedSize/totalSize);
		printf("\r|");
		for(i=0;i<20;i++) {
			if(percent/5 > i)
				printf("=");
			else if(percent/5 == i)
					printf(">");
			else printf("-");
		}
		printf("| %d:%02d/%d:%02d (%d%%)",secint/60, secint-60*(secint/60), (int)totalTime/60, (int)totalTime-60*((int)totalTime/60), percent);
		fflush(stdout);
		currentTime = secint;
	}
	
	return err;
}
